from setuptools import setup

setup(
    name='BiliDownloader',
    version='0.0.4',
    packages=['src', 'src.bili_downloader'],
    url='https://github.com/nkjcqvcpi/BiliDownloader',
    license='GPL',
    author='Nkjcqvcpi',
    author_email='lhtt100@gmail.com',
    description='A Downloader for Bili',
)
